/*
 * GDSII-Converter Python Plugin
 * Copyright (C) 2019  Mario Hüttel <mario.huettel@gmx.net>
 *
 * This file is part of GDSII-Converter.
 *
 * GDSII-Converter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * GDSII-Converter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GDSII-Converter.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @defgroup python-plugin Python Plugin for External Renderer
 * @ingroup plugins
 * This is a plugin for calling an external python script as renderer
 * @addtogroup python-plugin
 * @{
 */

#include <stdio.h>
#include <glib.h>
#include <unistd.h>
#include <Python.h>

#include <gds-render/gds-utils/gds-types.h>
#include <gds-render/output-renderers/external-renderer-interfaces.h>

/**
 * @brief Global variable for loaded module. This is not very nice.
 * @warning This has to be changed, if this api wants to be re-entrant.
 */
static PyObject *p_module = NULL;

static int check_file_exists(const char *path)
{
	FILE *file;
	int ret = -1;

	file = fopen(path, "rb");
	if (file) {
		fclose(file);
		ret = 0;
	}

	return ret;
}

int FUNC_DECL(EXTERNAL_LIBRARY_RENDER_FUNCTION)(struct gds_cell *toplevel, GList *layer_info_list, const char *output_file_name, double scale)
{
	int ret = 0;
	PyObject *p_render_func;
	PyObject *p_return_value;

	if (!toplevel)
		return -1000;
	if (!p_module)
		return -2000;

	printf("Rendering %s\n", toplevel->name);

	p_render_func = PyObject_GetAttrString(p_module, "test_func");
	if (!p_render_func && !PyCallable_Check(p_render_func)) {
		if (PyErr_Occurred())
			PyErr_Print();
		else
			fprintf(stderr, "Function not found in python module\n");

		ret = -1;
		goto return_value;
	}

	p_return_value = PyObject_CallObject(p_render_func, NULL);
	if (p_return_value) {
		printf("Result of call: %ld\n", PyLong_AsLong(p_return_value));
		Py_DECREF(p_return_value);
	}

	Py_XDECREF(p_render_func);

return_value:
	return ret;
}

int FUNC_DECL(EXTERNAL_LIBRARY_INIT_FUNCTION)(const char *params, const char *version)
{
	int ret = 0;
	PyObject *p_name;
	PyObject *p_sys_path;
	PyObject *p_cwd_string;
	char cwd[256];

	if (!params || !version)
		return -1000;

	printf("Init with params: %s\ngds-render version: %s\n", params, version);
	Py_Initialize();

	if (!getcwd(cwd, sizeof(cwd))) {
		return -2;
	}

	p_sys_path = PySys_GetObject("path");
	p_cwd_string = PyUnicode_FromString(cwd);
	PyList_Append(p_sys_path, p_cwd_string);
	Py_DECREF(p_cwd_string);

	/* This renderer has only one parameter: The name of the file */
	/*if (check_file_exists(params))
		return -1001;
	*/

	p_name = PyUnicode_DecodeFSDefault(params);
	p_module = PyImport_Import(p_name);
	Py_DECREF(p_name);

	if (!p_module) {
		PyErr_Print();
		fprintf(stderr, "Failed to load %s\n", params);
		ret = -1;
		goto return_value;
	}

return_value:
	return ret;
}

int FUNC_DECL(EXTERNAL_LIBRARY_FINALIZE_FUNCTION)(void)
{
	int ret;

	printf("Finalizing\n");

	if (p_module)
		Py_DECREF(p_module);

	ret = Py_FinalizeEx();
	return ret;
}

/** @} */
